#include "myInternetSession.h"
#include "nsStringAPI.h"
#include <sstream>

myInternetSession::myInternetSession(UploadObserver *observer, bool bStatusUpdate, LPCTSTR pstrAgent,
									 DWORD_PTR dwContext,
									 DWORD dwAccessType,
									 LPCTSTR pstrProxyName,
									 LPCTSTR pstrProxyBypass,
									 DWORD dwFlags) : m_bStatusUpdate(bStatusUpdate), 
									 CInternetSession(pstrAgent, dwContext, dwAccessType, pstrProxyName, pstrProxyBypass, dwFlags)
{
	m_Observer = observer;
}

void myInternetSession::OnStatusCallback(
	DWORD_PTR dwContext,
	DWORD dwInternetStatus,
	LPVOID lpvStatusInformation,
	DWORD dwStatusInformationLength 
	)
{
	try {
		AFX_MANAGE_STATE( AfxGetAppModuleState( ) );
		if (m_Observer && m_bStatusUpdate)	{
			std::ostringstream out;
			out << dwInternetStatus << " : " ;
			bool bSthgToLog = false;
			switch (dwInternetStatus) {
			case INTERNET_STATUS_RESOLVING_NAME:
				bSthgToLog = true;
				out << "Looking up the IP address of ";
				if(dwStatusInformationLength > 0 && lpvStatusInformation) {
					for (int index = 0; index<dwStatusInformationLength; ++index) {
						out << ((char*)lpvStatusInformation)[2*index];
					}
				}
				break;
			case INTERNET_STATUS_NAME_RESOLVED :
				bSthgToLog = true;
				out << "Successfully found the IP address : ";
				if(dwStatusInformationLength > 0 && lpvStatusInformation) {
					for (int index = 0; index<dwStatusInformationLength; ++index) {
						out << ((char*)lpvStatusInformation)[index];
					}
				}
				break;
			case INTERNET_STATUS_CONNECTING_TO_SERVER :
				bSthgToLog = true;
				out << "Connecting to the socket address (SOCKADDR) pointed to by ";
				if(dwStatusInformationLength > 0 && lpvStatusInformation) {
					for (int index = 0; index<dwStatusInformationLength; ++index) {
						out << ((char*)lpvStatusInformation)[index];
					}
				}
				break;
			case INTERNET_STATUS_CONNECTED_TO_SERVER :
				bSthgToLog = true;
				out << "Successfully connected to the socket address (SOCKADDR) pointed to by ";
				if(dwStatusInformationLength > 0 && lpvStatusInformation) {
					for (int index = 0; index<dwStatusInformationLength; ++index) {
						out << ((char*)lpvStatusInformation)[index];
					}
				}
				break;
			case INTERNET_STATUS_SENDING_REQUEST :
				bSthgToLog = false;
				out << "Sending the information request to the server.";
				break;
			case INTERNET_STATUS_REQUEST_SENT :
				bSthgToLog = false;
				out << "Successfully sent the information request to the server.";
				break;
			case INTERNET_STATUS_RECEIVING_RESPONSE :
				bSthgToLog = true;
				out << "Waiting for the server to respond to a request.";
				break;
			case INTERNET_STATUS_RESPONSE_RECEIVED :
				bSthgToLog = true;
				out << "Successfully received a response from the server.";
				break;
			case INTERNET_STATUS_CLOSING_CONNECTION :
				bSthgToLog = true;
				out << "Closing the connection to the server.";
				break;
			case INTERNET_STATUS_CONNECTION_CLOSED :
				bSthgToLog = true;
				out << "Successfully closed the connection to the server.";
				break;
			case INTERNET_STATUS_HANDLE_CREATED :
				bSthgToLog = true;
				out << "it has created a new handle.";
				break;
			case INTERNET_STATUS_HANDLE_CLOSING :
				bSthgToLog = true;
				out << "Successfully terminated this handle value.";
				break;
			case INTERNET_STATUS_COOKIE_SENT:
				bSthgToLog = true;
				out << "number of cookies that were either sent or suppressed :" <<*((DWORD*)lpvStatusInformation);
				break;
			default:
				bSthgToLog = false;
				break;
			}
			if (bSthgToLog) {
				CString stringInfo = out.str().c_str();
				m_Observer->OnStatus(nsString(stringInfo));
			}
		}
	}
	catch(...)
	{
	}
}

myInternetSession::~myInternetSession(void)
{
}
